from FatxFast.gui.hud.frame import Frame
from FatxFast.gui.hud.globalhud import GlobalHud
from FatxFast.gui.hud.gameframe import GameFrame
from FatxFast.gui.boxwidget import BoxWidget
from FatxFast.graphics import SCREENSIZE
from FatxFast.tilemap.editor.tile import TILEWIDTH, TILEHEIGHT

class GameHud(Frame):
    
    def __init__(self, game, **kwargs):
        super(GameHud, self).__init__(game, **kwargs)
        self.game.add_widget("game", self)
        self._hud = GlobalHud(game,**self.fetch_template("gamebar"))
        self.add_widget("global-hud", self._hud)
        self._player_count = 0

    def add_player(self, player, camera):
        attr = self.fetch_template("gamehud")
        attr["container"] = self
        widget = GameFrame(self.game, player, camera, **attr)
        widget.name = "hud-"+str(self._player_count)
        self.add_widget(widget.name, widget) 
        self._player_count += 1
        self._fit_gameframes()
        if self._player_count > 1:
            self._add_separator(self._player_count-1)

    def _add_separator(self, index):
        attr = self.fetch_template("separator")
        attr["container"] = self
        frame = self.get_widget("hud-"+str(index))
        attr["x"] = frame.rect.width*index
        attr["height"] = frame.rect.height
        attr["width"] = TILEWIDTH
        widget = BoxWidget(**attr)
        widget.name = "separator-"+str(index)
        self.add_widget(widget.name, widget)

    def _fit_gameframes(self):
        for index in xrange(0, self._player_count):
            frame = self.get_widget("hud-"+str(index))
            frame.rect.width = \
                (SCREENSIZE[0]-((self._player_count-1)*TILEWIDTH)) \
                    /self._player_count
            frame.rect.x = index*(frame.rect.width+TILEWIDTH)
    
    def remove_players(self):
        for index in xrange(0, self._player_count):
            self.delete_widget("hud-"+str(index))
            self.delete_widget("separator-"+str(index))
        self._player_count = 0

