# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.2.3"


import pygame
from FatxFast.objects.entity import GameEntity
from FatxFast.objects.moveable_entity import MoveableEntity

# Buff IDs

COFFEE_BUFF_ID = 513
WARP_DEBUFF_ID = 514
MINE_BUFF_ID = 515

class Buff(GameEntity):
    
    def __init__(self, duration, **kwargs):
        super(Buff, self).__init__(colorkey="blue", **kwargs)
        self.duration = duration

    def update(self, dt): 
        self.duration -= dt

class CharacterBuff(Buff):
    
    def __init__(self, character, duration, **kwargs):
        super(CharacterBuff, self).__init__(duration, **kwargs)
        self.character = character

class WarpDebuff(CharacterBuff):
    def __init__(self, character, duration, **kwargs):
        super(WarpDebuff, self).__init__(character, duration, 
            imagename="warpdebuff.png", entity_id=WARP_DEBUFF_ID, **kwargs)

class SpeedBuff(CharacterBuff):
    
    def __init__(self, character, duration, **kwargs):
        super(SpeedBuff, self).__init__(character, duration, 
            imagename="coffee.png", entity_id=COFFEE_BUFF_ID)
        self.speedmultiplier = 1.3

    def update(self, dt):
        super(SpeedBuff, self).update(dt)
        if self.duration > 0:
            self.character.maxspeed = MoveableEntity.maxspeed\
                *self.speedmultiplier
        else:
            self.character.maxspeed = MoveableEntity.maxspeed

"""class SpeedDebuff(CharacterBuff):

    def __init__(self, character, duration, **kwargs):
        super(SpeedDebuff, self).__init__(character, duration, 
            imagename="mine.png", entity_id=MINE_BUFF_ID)

    def update(self, dt):
        super(SpeedDebuff, self).update(dt)
        if self.duration > 0:
            self.character.maxspeed = TILEWIDTH*0.4
        else:
            self.character.maxspeed = TILEWIDTH*0.65"""

BUFF_CLASSES = {
            WARP_DEBUFF_ID: WarpDebuff,
            COFFEE_BUFF_ID: SpeedBuff}

