
# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.2.3"


from FatxFast.scenes.basescene import Scene
from FatxFast.scenes.sceneeditors import AskLoadSceneEditor, GameEndSceneEditor
import pygame

class PopupScene(Scene):
    
    def __init__(self, manager, **kwargs):
        super(PopupScene, self).__init__(manager, **kwargs)
        if "text" in kwargs:
            self.get_widget("popuptext").text = kwargs["text"]

    def key_pressed(self, key, utf, mod):
        if key == pygame.K_RETURN or key == pygame.K_ESCAPE:
            self.manager.pop_scene()
        else:
            super(PopupScene, self).key_pressed(key, utf, mod)

class GameEndScene(PopupScene):
    
    def __init__(self, manager, **kwargs):
        super(GameEndScene, self).__init__(manager, **kwargs)
        self.editor = GameEndSceneEditor(self)
        self.editor.connect_button_links()
        self._played_mode = kwargs.get("played_mode")

    def key_pressed(self, key, utf, mod):
        super(GameEndScene, self).key_pressed(key, utf, mod)
        if key == pygame.K_r:
            self.restart_mode()

    def restart_mode(self):
        self.manager.pop_scene()
        self.manager.push_scene_by_name("gamescene", 
            mode=self._played_mode)

class AskLoadScene(PopupScene):    
    
    def __init__(self, manager, target, **kwargs):
        super(AskLoadScene, self).__init__(manager)
        self.editor = AskLoadSceneEditor(self)
        self.editor.connect_button_links()
        self.target = int(target)
        self.gamescene = kwargs.get("gamescene")
        self.save = self.manager.game.savemanager.get_save(self.target)
        if self.save:
            date = self.save.game["date"]
        else:
            date = "Empty" 
        self.get_widget("saveinfo").text += date 

    def load_save(self):
        self.manager.push_scene_by_name("gamescene", 
            save=self.save)
    
    def save_to_save(self): 
        if self.gamescene:
            self.gamescene.save_game(self.target)
        self.manager.pop_scene()

    def key_pressed(self, key, utf, mod):
        super(AskLoadScene, self).key_pressed(key, utf, mod)
        if key == pygame.K_s:
            self.save_to_save()
        if key == pygame.K_l:
            self.load_save()
                
class HotkeyScene(PopupScene):
    
    def __init__(self, manager, **kwargs):
        super(HotkeyScene, self).__init__(manager, **kwargs)
        self.button = kwargs.get("button")
        self.config = kwargs.get("config")
        
    def change_hotkey(self, new_hotkey):
        self.config.change_hotkey(self.button, new_hotkey)
        
    def key_pressed(self, key, utf, mod):
        if key == pygame.K_ESCAPE or key == pygame.K_RETURN:
            super(HotkeyScene, self).key_pressed(key, utf, mod)
        else:
            self.change_hotkey(key)
            self.manager.pop_scene()
